#!/usr/bin/python2

####################################################################################################################
##### Script runs product udpates
#####
##### Syntax: update-manager.py --mode <checkupdate|update>
##### Example: update-manager.py --mode update
##### 
####################################################################################################################

from subprocess import call, check_output, Popen, PIPE
import string
import random
import os, shutil, logging, sys, re, glob
from optparse import OptionParser, SUPPRESS_HELP 
import urllib , urllib2, json

CONST = None
FORWARD_SLASH="/"
YUM='/usr/bin/yum'
DNF='/usr/bin/dnf'
UPDALT='/usr/sbin/update-alternatives'
GREP='/usr/bin/grep'
AWK='/usr/bin/awk'
PING='/bin/ping'
OSUPDATE_REPO_CONFIG='/etc/yum.repos.d/osupdate.repo'
SPP_REPO_CONFIG='/etc/yum.repos.d/spp.repo'

class LessThenFilter(logging.Filter):
    def __init__(self, level):
	self._level = level
	logging.Filter.__init__(self)

    def filter(self, rec):
        return rec.levelno < self._level

def setup():
    
    global logger

    logger = logging.getLogger('update')
    logger.setLevel(logging.DEBUG)	

    # create a console handler
    cHandler = logging.StreamHandler(sys.stdout)
    cHandler.setLevel(logging.INFO)
    cHandler.setFormatter(logging.Formatter('%(message)s'))
    cHandler.addFilter(LessThenFilter(logging.WARNING))
    logger.addHandler(cHandler)


    # create a console handler for stderr
    ceHandler = logging.StreamHandler()
    ceHandler.setLevel(logging.ERROR)
    ceHandler.setFormatter(logging.Formatter('%(message)s'))
    #ceHandler.addFilter(LessThenFilter(logging.WARNING))
    logger.addHandler(ceHandler)

    # create a file handler
    fHandler = logging.FileHandler(options.log)
    fHandler.setLevel(logging.DEBUG)
    fHandler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s' , "%Y-%m-%d %H:%M:%S" ))
    logger.addHandler(fHandler)


def main():

        if options.yumclean: yumcleanall()

        if options.group is None : 
            group=repo.get_group_name()
        else: group=options.group

        if options.mode =='checkupdate':  check_update(group)

        if options.mode =='checkupgrade':  check_upgrade()

        if options.mode =='updateos':
            if options.package is not None:

                update_or_install_package(options.package, False, options.skip_gpg_check)
            else:
                update_or_install_package('')

        if options.mode =='install': 
            if options.package is not None :
                install_package(options.package, True)

        if options.mode =='update':            
            
            if options.package is not None : 
                update_or_install_package(options.package, True)
            
            elif options.group is None : 

                all_groups=group
                if is_update_available('endeavour-platform'): all_groups += ' endeavour-platform'
                # special handle for vsnap  , the group name is 'vsnap' 
                if is_update_available('vsnap'): 

                    update_or_install_package("vsnap-dist", True)

                    run_vsnap_dist_installer()


                #update_or_install_package('spp-guestappvm', True)
                #update_or_install_package('spp-nmt', True)

                update_group( all_groups , False)

            else:
               # if  group == 'spp':
               #     # fix update for disconnected from the 'spp' group spp-guestappvm
               #     update_or_install_package('spp-guestappvm', True)
               #     update_or_install_package('spp-nmt', True)

                update_group( group )

        if options.mode =='checkyumserver': checkyumserver()
        
        if options.mode =='updaterepourl': 
		if options.repourl is None:
                    logger.warning ("The '--updaterepourl' option expects url , pass with '--repourl'" )

                    failed_exit()
		else:
                    update_repo_url(options.repourl)

        if options.mode =='updatereposppv':

            if options.spp_version is None:
                logger.warning ("The '--updatereposppv' option expects version , pass with '--spp-version'" )

                failed_exit()
            else:
                update_repo_env_spp_version(options.spp_version)
                yumcleanall()


def clean_exit():
        logger.info('Script completed successfully, Exiting!')
        sys.exit(0)

def failed_exit():
        #logger.debug('Script completed with failures ......')
        sys.exit(1)

#
# updates /etc/yum/vars/spp_version file
#
def update_repo_env_spp_version(version):
    logger.info('Update environmet variable for baseurl: ' + version)

    with open('/etc/yum/vars/spp_version' , 'w') as file:
        file.write(version)



def update_repo_url(repourl):
        logger.info('Repo url to update ' + repourl)

    	command = 'sed -i -e s#^baseurl=.*#baseurl=' + repourl + '#g ' + repo.get_config_file() 
    
	logger.info('Running command: ' + command)
    
	pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
	out, err = pobj.communicate()
    
	#logger.debug('Command Returncode : ' + str(pobj.returncode))

	if pobj.returncode == 0:
		if out:
			logger.info(out)            
			clean_exit()
        else:
		if out:
			logger.info(out)

		if err:
			logger.error(err)                
		
		failed_exit()


#
#  for internal use , does not exit
#    returns True if update is available, otherwise False
# 
def is_update_available(group):
    packages = get_packages_in_group( group )

    if  packages:
        s = ' '.join(packages)

        command = YUM + ' -q --disablerepo=* --enablerepo=' + repo.get_id() + ' check-update ' + s

        logger.info('Running command: ' + command)
        pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
        out, err = pobj.communicate()
        if pobj.returncode == 100: 
            return True
        else:
            logger.info('No updates available for packages ' + s)

    return False


# is_ping(address)
#    arguments address
def is_ping(address):
    address = '8.8.8.8'
    retVal = False

    logger.debug("Ping address supplied: " + address)

    if address == '':
       return False

    command = PING + ' -c 1 ' + address + ' > /dev/null 2>&1'  
    logger.debug("Ping check: " + command)
    rc = call([command], shell=True)
    if rc == 0:
      logger.debug('Ping Success')
      retVal = True
    else:
      logger.debug('Ping Failed')

    return retVal

# get_repo_server(repofile)
#       arguments full path to repo config
def get_repo_server(repofile):
   retVal = '';
   command = GREP + " baseurl " + repofile + " | " + AWK + " -F'/' '{print $3}'"
   logger.debug('Running command: ' + command)

   return check_output([command], shell=True).strip()

#
# check_update()
#   arguments: group
#
def check_update(group):
 
    logger.info('Checking available update for group: ' + group )

    packages = get_packages_in_group( group )
    if not packages:
	logger.error('No packages in the group ' + group)
	failed_exit()

    s = ' '.join(packages)

    command = YUM + ' -q --disablerepo=* --enablerepo=' + repo.get_id() + '  check-update ' + s

    logger.info('Running command: ' + command)
    
    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
    out, err = pobj.communicate()
    
    #logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 100:   # update is available
        if out:
            logger.info(out)            
            clean_exit()
    else:
        if out:
            logger.info(out)

        if err:
            logger.error(err)

        logger.error('Attempt to upgrade to an older or current version of IBM Spectrum Protect Plus is not permitted.  Upgrade is terminating.')

        failed_exit()

#
# checks update from current repo or looking into 'nvr' tag file ( upgrade ) 
#  returns list with one or two elements
# Example:
# [{ "version": "10.1.4", build: "380"}, { "version": "10.1.6", "build": ""}]
#
def check_upgrade():


    update_version = {}
    upgrade_version = {}
    update_upgrade_list = []

    update_version=repo.get_spp_version()

    if  update_version:  # if not empty and update is available
        update_upgrade_list.append(update_version)

    nsv=repo.get_next_supported_version()



    if  nsv :
        upgrade_version["version"]=nsv
        upgrade_version["build"]=''
        update_upgrade_list.append(upgrade_version )


    update_upgrade_list_json=json.dumps(update_upgrade_list)

    #print (update_upgrade_list_json) 
    logger.info( update_upgrade_list_json )



#
#
#
def add_new_packages_in_spp_group():

    update_or_install_package('spp-nmt', True)
    update_or_install_package('spp-k8s', True)
    update_or_install_package('spp-plugins-cm', True)
    update_or_install_package('spp-plugins-awsebs', True)
    update_or_install_package('spp-plugins-awsec2', True)
    # Replace openjdk with IBM java
    check_installed_openjdk()
    check_installed_ibmjava()

    yum_groups_mark_packages('spp', 'spp-guestappvm spp-nmt spp-k8s spp-plugins-cm spp-plugins-awsebs spp-plugins-awsec2 vadp vadp-inst')
    yum_groups_mark_packages('adminui', 'spp-upst')

#
# update_group()
#
def update_group(group, continue_flag=False):

    if 'spp' in group: add_new_packages_in_spp_group()
    
    yum_group_info(group)

    command = YUM + ' --disablerepo=* --enablerepo=' + repo.get_id() + ' -y groupupdate ' + group

    logger.info('Running command: ' + command)
    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')

    while True:
        line = pobj.stdout.readline()
        logger.info(line)
        if line == '' and pobj.poll() != None:
            break
            
    #logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if not continue_flag: clean_exit()
    else:

    	while True:
        	line = pobj.stderr.readline()
	        logger.error(line)
        	if line == '' and pobj.poll() != None:
	            break
        
        failed_exit()


#
#  check_installed_package()
#    returns true if package is installed
#
def check_installed_package(package):

    cmd=("rpm -q " + package )
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()

    #print ( 'debug  return code: ' + str(p.returncode) ) 
    
    if p.returncode == 0 : return True 
    else: return False

#
#  install command
#
def install_package( package, continue_flag=False):

    command = YUM + ' --disablerepo=* --enablerepo=' + repo.get_id() + ' -y install ' + package
    execute_yum_command(command,continue_flag)


#
# update_package()
#   update all if package is empty 
#
def update_or_install_package( package, continue_flag=False, skip_gpg_check=False):

    YUM_OPTION=''    

    if skip_gpg_check: 
        YUM_OPTION+=' --nogpgcheck '

    if package=='':
        # only updates are allowed

        logger.info('Updating all packages from ' + repo.get_id() + ' repository' )
        command = YUM  + YUM_OPTION + ' --disablerepo=* --enablerepo=' + repo.get_id() + ' -y update '

    else:

        # package is not found, install the package
        if check_installed_package(package):
            logger.info('Updating package:  ' + package )
            command = YUM  + YUM_OPTION + ' --disablerepo=* --enablerepo=' + repo.get_id() + ' -y update ' + package

        else:
            logger.info('Installing package:  ' + package )
            command = YUM  + YUM_OPTION + ' --disablerepo=* --enablerepo=' + repo.get_id() + ' -y install ' + package

    execute_yum_command(command,continue_flag)


def execute_yum_command( command , continue_flag=False):

    logger.info('Running command: ' + command)
    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')

    while True:
        line = pobj.stdout.readline()
        logger.info(line)
        if line == '' and pobj.poll() != None:
            break
            
    #logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if not continue_flag: clean_exit()
    else:

    	while True:
        	line = pobj.stderr.readline()
	        logger.error(line)
        	if line == '' and pobj.poll() != None:
	            break
        
        failed_exit()

#
# run_vsnap_dist_installer()
#    runs /opt/vsnap-dist/vsnap-dist-*.run
#
def run_vsnap_dist_installer():

    list=glob.glob('/opt/vsnap-dist/vsnap-dist-*.run')

    if list: # installer exists
        installer = list[0]
        command = installer + '  -- nostart'
        pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')

        logger.info('Running command: ' + command)

        while True:
            line = pobj.stdout.readline()
            logger.info(line)
            if line == '' and pobj.poll() != None:
                break

        #logger.debug('Command Returncode : ' + str(pobj.returncode))

        if pobj.returncode == 0:
            logger.info('Successful installation of vsnap distribution')
        else:

            while True:
                    line = pobj.stderr.readline()
                    logger.error(line)
                    if line == '' and pobj.poll() != None:
                        break


#
# get_packages_in_group()
#
def get_packages_in_group(group):

    packages = []

    cmd = YUM  + ' -q --disablerepo=* --enablerepo=' + repo.get_id() + ' groupinfo ' + group

    logger.info('Running command: ' + cmd)
    start_read=0
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    with p.stdout:
        for line in iter(p.stdout.readline, b''):
            if line.startswith(" Mandatory Packages:"):
                start_read=1
                continue
            if line.startswith(" Installed Packages:"): continue

            if start_read: packages.append(line.strip().strip('=').strip('-').strip('+') )
    p.wait() # wait for the subprocess to exit

    return packages
    

def checkyumserver():
    status=repo.get_http_server_code()
    if str(status) == '200': return 0     
    else:
        failed_exit() 


def yumcleanall():
    logger.info('Cleaning yum cache...with expire-cache')
    cmd=[ YUM, '--enablerepo=*', 'clean', 'all', 'expire-cache']
    Popen(cmd).wait()


def yum_convert_groups_to_objects():
    logger.info('Converting yum packages to objects.')    
    cmd=[ YUM, '--disablerepo=*' ,'--enablerepo=' + repo.get_id() , 'groups', '--setopt=group_command=objects', 'mark-convert']
    Popen(cmd).wait()


def yum_group_info(group):
    logger.info('Group info for ' + group)    
    cmd=[ YUM, '--disablerepo=*' ,'--enablerepo=' + repo.get_id() , 'groupinfo' , group ]
    Popen(cmd).wait()


def check_installed_openjdk():
    logger.info('Checking for previously installed openjdk')
    cmd = 'rpm -q java-1.8.0-openjdk java-1.8.0-openjdk-devel java-1.8.0-openjdk-headless'
    logger.info('Running command: ' + cmd)
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()
    print (stdout)
    print (stderr)
    print ( 'debug  return code: ' + str(p.returncode) )
    logger.info( 'stdout value: ' + stdout )
    logger.info( 'stderr value: ' + stderr )
    logger.info( 'debug  return code: ' + str(p.returncode) )
    if p.returncode == 0 :
        rcmd=['rpm', '-e', 'java-1.8.0-openjdk', 'java-1.8.0-openjdk-devel', 'java-1.8.0-openjdk-headless']
        Popen(rcmd).wait()
    else:
        logger.info('No installed openjdk detected.')


def check_installed_ibmjava():
    logger.info('Checking for previously installed IBM java')
    cmd = 'rpm -q ibm-java-x86_64-sdk'
    logger.info('Running command: ' + cmd)
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()
    print (stdout)
    print (stderr)
    print ( 'debug  return code: ' + str(p.returncode) )
    logger.info( 'stdout value: ' + stdout )
    logger.info( 'stderr value: ' + stderr )
    logger.info( 'debug  return code: ' + str(p.returncode) )
    if p.returncode == 0 :
        logger.info('IBM Java detected.')
    else:
        logger.info('Installing IBM Java.')
        update_or_install_package('ibm-java-x86_64-sdk', True)
        updatejavaalt()


def updatejavaalt():
    logger.info('Updating java alternatives file...')
    cmd=[ UPDALT, '--install', '/usr/bin/java', 'java', '/opt/ibm/java-x86_64-80/bin/java', '1', \
                  '--slave', '/usr/bin/jjs', 'jjs', '/opt/ibm/java-x86_64-80/bin/jjs', \
                  '--slave', '/usr/bin/keytool', 'keytool', '/opt/ibm/java-x86_64-80/bin/keytool', \
                  '--slave', '/usr/bin/pack200', 'pack200', '/opt/ibm/java-x86_64-80/bin/pack200', \
                  '--slave', '/usr/bin/policytool', 'policytool', '/opt/ibm/java-x86_64-80/bin/policytool', \
                  '--slave', '/usr/bin/rmid', 'rmid', '/opt/ibm/java-x86_64-80/bin/rmid', \
                  '--slave', '/usr/bin/rmiregistry', 'rmiregistry', '/opt/ibm/java-x86_64-80/bin/rmiregistry', \
                  '--slave', '/usr/bin/tnameserv', 'tnameserv', '/opt/ibm/java-x86_64-80/bin/tnameserv', \
                  '--slave', '/usr/bin/unpack200', 'unpack200', '/opt/ibm/java-x86_64-80/bin/unpack200']
    Popen(cmd).wait()
    logger.info('Setting java alternatives file...')
    cmd=[ UPDALT, '--set', 'java', '/opt/ibm/java-x86_64-80/bin/java']
    Popen(cmd).wait()
    logger.info('Updating javac alternatives file...')
    cmd=[ UPDALT, '--install', '/usr/bin/javac', 'javac', '/opt/ibm/java-x86_64-80/bin/javac', '1', \
                  '--slave', '/usr/bin/appletviewer', 'appletviewer', '/opt/ibm/java-x86_64-80/bin/appletviewer', \
                  '--slave', '/usr/bin/ControlPanel', 'ControlPanel', '/opt/ibm/java-x86_64-80/bin/ControlPanel', \
                  '--slave', '/usr/bin/extcheck', 'extcheck', '/opt/ibm/java-x86_64-80/bin/extcheck', \
                  '--slave', '/usr/bin/idlj', 'idlj', '/opt/ibm/java-x86_64-80/bin/idlj', \
                  '--slave', '/usr/bin/jar', 'jar', '/opt/ibm/java-x86_64-80/bin/jar', \
                  '--slave', '/usr/bin/jarsigner', 'jarsigner', '/opt/ibm/java-x86_64-80/bin/jarsigner', \
                  '--slave', '/usr/bin/javadoc', 'javadoc', '/opt/ibm/java-x86_64-80/bin/javadoc', \
                  '--slave', '/usr/bin/javah', 'javah', '/opt/ibm/java-x86_64-80/bin/javah', \
                  '--slave', '/usr/bin/javap', 'javap', '/opt/ibm/java-x86_64-80/bin/javap', \
                  '--slave', '/usr/bin/javaw', 'javaw', '/opt/ibm/java-x86_64-80/bin/javaw', \
                  '--slave', '/usr/bin/javaws', 'javaws', '/opt/ibm/java-x86_64-80/bin/javaws', \
                  '--slave', '/usr/bin/jconsole', 'jconsole', '/opt/ibm/java-x86_64-80/bin/jconsole', \
                  '--slave', '/usr/bin/jcontrol', 'jcontrol', '/opt/ibm/java-x86_64-80/bin/jcontrol', \
                  '--slave', '/usr/bin/jdb', 'jdb', '/opt/ibm/java-x86_64-80/bin/jdb', \
                  '--slave', '/usr/bin/jdeps', 'jdeps', '/opt/ibm/java-x86_64-80/bin/jdeps', \
                  '--slave', '/usr/bin/jdmpview', 'jdmpview', '/opt/ibm/java-x86_64-80/bin/jdmpview', \
                  '--slave', '/usr/bin/jrunscript', 'jrunscript', '/opt/ibm/java-x86_64-80/bin/jrunscript', \
                  '--slave', '/usr/bin/native2ascii', 'native2ascii', '/opt/ibm/java-x86_64-80/bin/native2ascii', \
                  '--slave', '/usr/bin/rmic', 'rmic', '/opt/ibm/java-x86_64-80/bin/rmic', \
                  '--slave', '/usr/bin/schemagen', 'schemagen', '/opt/ibm/java-x86_64-80/bin/schemagen', \
                  '--slave', '/usr/bin/serialver', 'serialver', '/opt/ibm/java-x86_64-80/bin/serialver', \
                  '--slave', '/usr/bin/wsgen', 'wsgen', '/opt/ibm/java-x86_64-80/bin/wsgen', \
                  '--slave', '/usr/bin/wsimport', 'wsimport', '/opt/ibm/java-x86_64-80/bin/wsimport', \
                  '--slave', '/usr/bin/xjc', 'xjc', '/opt/ibm/java-x86_64-80/bin/xjc', \
                  '--slave', '/usr/bin/ikeycmd', 'ikeycmd', '/opt/ibm/java-x86_64-80/jre/bin/ikeycmd', \
                  '--slave', '/usr/bin/ikeyman', 'ikeyman', '/opt/ibm/java-x86_64-80/jre/bin/ikeyman', \
                  '--slave', '/usr/bin/j9vm', 'j9vm', '/opt/ibm/java-x86_64-80/jre/bin/j9vm', \
                  '--slave', '/usr/bin/java-rmi.cgi', 'java-rmi.cgi', '/opt/ibm/java-x86_64-80/jre/bin/java-rmi.cgi', \
                  '--slave', '/usr/bin/jextract', 'jextract', '/opt/ibm/java-x86_64-80/jre/bin/jextract', \
                  '--slave', '/usr/bin/kinit', 'kinit', '/opt/ibm/java-x86_64-80/jre/bin/kinit', \
                  '--slave', '/usr/bin/klist', 'klist', '/opt/ibm/java-x86_64-80/jre/bin/klist', \
                  '--slave', '/usr/bin/ktab', 'ktab', '/opt/ibm/java-x86_64-80/jre/bin/ktab']
    Popen(cmd).wait()
    logger.info('Setting javac alternatives file...')
    cmd=[ UPDALT, '--set', 'javac', '/opt/ibm/java-x86_64-80/bin/javac']
    Popen(cmd).wait()
    mkdcmd=['mkdir', '-p', '/usr/lib/jvm/jre/bin']
    Popen(mkdcmd).wait()
    lnkcmd=['ln', '-s', '/opt/ibm/java-x86_64-80/jre/bin/java', '/usr/lib/jvm/jre/bin/java']
    Popen(lnkcmd).wait()
    mkdjcmd=['mkdir', '-p', '/usr/java']
    Popen(mkdjcmd).wait()
    lnkjcmd=['ln', '-s', '/opt/ibm/java-x86_64-80', '/usr/java/latest']
    Popen(lnkjcmd).wait()


def yum_groups_mark_packages(group,packages):
    logger.info('yum_groups_mark_packages  for ' + group)
    package_list=packages.split()
    cmd=[ DNF, 'mark', 'group' ] + package_list
    logger.info(cmd)    
    Popen(cmd).wait()

 
#       
# Represents yum repository
#
class Repo:
    'Class for yumrepository'

    def __init__(self,repo='localrepo'):

        self.group_name  = 'spp'

        self.id = repo
        cmd = YUM  + ' repoinfo ' + repo

        p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
        stdout, stderr = p.communicate()

        for line in stdout.splitlines():
            if line.startswith("Repo-name"):  self.name    = line.split(':')[1]
            if line.startswith("Repo-status"):  self.status  = line.split(':')[1]
            if line.startswith("Repo-baseurl"):  
                self.baseurl = line.split(' : ')[1]
                if not self.baseurl.endswith('/'): self.baseurl+='/'



        #print (self.baseurl )


 
    def get_id(self):
        return self.id
 
    def get_baseurl(self):
        return self.baseurl

    def get_group_name(self):
        return self.group_name

    def get_config_file(self):
	return self.config

    def get_http_server_code(self):

        if self.baseurl.startswith("http"):
            code=urllib.urlopen(self.baseurl).getcode()

            logger.debug('HTTP status code (' + self.baseurl + ')  : ' + str(code))

        else:
            # check internet connection anyway
            code=urllib.urlopen('http://www.google.com').getcode()
            logger.info('Local repository ' + self.id + ' wad detected , checking internet connectivity anyway: ' + str(code))

        return code

    #
    # reads 'nsv' file, supplied with image or updated manually
    #
    def get_next_supported_version(self):
        
        version=''
        target_url=self.baseurl + 'nsv'

        try:
            f = urllib2.urlopen(target_url)
            read_data=f.read()
            version=read_data.strip()
        except: 
            logger.warning ('Failed to open url ' + target_url + ' , or file does not exist')

            
        return version



    #
    # resolve version from release package 
    # yum check-update returns: 
    #  spp-release.noarch  <version>  <reponame>
    # 
    # retrns dictionary:
    #     {'version': '999.0.0', 'build': '1228'}
    def get_spp_version(self):
        
        release_package='spp-release'

        dict = {}

        try:

            cmd = YUM  + ' -q --disablerepo=* --enablerepo=' + repo.get_id() + ' check-update  ' + release_package

            if options.verbosity > 1:
                logger.info('Running command: ' + cmd)

            p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
            stdout, stderr = p.communicate()
            
            m = re.match( r'.*' + release_package + '.\S+\s+(\S+).*', stdout , re.S)
            if(m):

                #logger.info('Version available on update: ' + m.group(1) )
                version,build=m.group(1).split('-')
                dict['version'] = version
                dict['build']   = build
                
            else:
                logger.warning('Can not detect version eligible for update :  ' + cmd)
                

        except Exception as e: 
            logger.warning(e)

        return dict




if __name__ == "__main__":

    version="1.8.3"
    parser = OptionParser()
    usage = "usage: %prog [options]"
    parser = OptionParser(usage=usage, version="%prog " + version)
    
    parser.add_option("-l","--log",
                      default='/data/log/adminconsole/update-manager.log',
                      dest='log',
                      help="Log file location"
                      " [ default: %default ]" )

    parser.add_option("--group",
                      dest='group',
                      help=SUPPRESS_HELP ) 

    parser.add_option("--package",
                      dest='package',
                      help=SUPPRESS_HELP ) 

    parser.add_option("-v", action="count", dest="verbosity", help=SUPPRESS_HELP)

    # undocumented option:  updatereposppv [update yum repo env vars for spp_version]
    parser.add_option("--mode",
                      help="supported mode: checkupdate, checkupgrade, update, updateos, checkyumserver, updaterepourl, install")


    parser.add_option("--yumclean", action="store_true", default=False,dest='yumclean', help=SUPPRESS_HELP )


    parser.add_option("--repourl",
		      dest='repourl',
		      help=SUPPRESS_HELP ) 

    parser.add_option("--skip-gpg-check",
                      action="store_true",
		      dest='skip_gpg_check',
                      default=False,
		      help=SUPPRESS_HELP ) 


    parser.add_option("--spp-version",
		      dest='spp_version',
		      help=SUPPRESS_HELP ) 


    (options, args) = parser.parse_args()

    if options.mode is None:
        print ("A mandatory option 'mode' is missing\n")
        parser.print_help()
        failed_exit()

    setup()    

    if options.verbosity  > 1:
        logger.info('+----------------------- STARTING UPDATE MANAGER --------------------------------+')
        logger.info(os.path.realpath(__file__) + ' called with options:')
        logger.info(options)

    if options.mode =='updateos' and os.path.exists('/etc/yum.repos.d/local.repo'):
        repo=Repo('osupdate')

    elif  os.path.exists('/etc/yum.repos.d/local.repo'):
        repo=Repo('localrepo')

    elif options.mode =='updateos' and os.path.exists(OSUPDATE_REPO_CONFIG) and is_ping(get_repo_server(OSUPDATE_REPO_CONFIG)):
        repo=Repo('osupdaterepo')

    elif os.path.exists(SPP_REPO_CONFIG) and is_ping(get_repo_server(SPP_REPO_CONFIG)):
        repo=Repo('spprepo')
    else:
        print ('Can not detect yum repository')
        failed_exit()

    main()
